#!/usr/bin/ruby
# $Id: htmlparser.rb 253 2010-12-28 19:24:17Z briane $

require 'open3'
require 'rexml/document'
require "./datatypes.rb"
include REXML

class HtmlParser

@tidy_location = ""
@cache_location_xml = ""
@cache_location_html = ""

def initialize
    # Test that "tidy" exists in the path
    @tidy_location = `which tidy`
    @tidy_location.chomp!
    raise "Unable to find the 'tidy' application. Do you have it installed?" if @tidy_location.length == 0
    
    # Create the cache location
    @cache_location_xml = './cache/xml'
    @cache_location_html = './cache/html'
    begin
        Dir.mkdir('./cache', 493) # 0755 permissions
    rescue
        # do nothing, probably already exists
    end
    begin
        Dir.mkdir('./cache/xml', 493) # 0755 permissions
    rescue
        # do nothing, probably already exists
    end
    raise "Unable to create #{@cache_location_xml} folder" if !File.exists?(@cache_location_xml) || !File.directory?(@cache_location_xml)
    begin
        Dir.mkdir('./cache/html', 493) # 0755 permissions
    rescue
        # do nothing, probably already exists
    end
    raise "Unable to create #{@cache_location_html} folder" if !File.exists?(@cache_location_html) || !File.directory?(@cache_location_html)
    
    @title_regex = Regexp.new('<h3>([^<]*)')
end

def Parse(wiki_article_data)
    if nil == wiki_article_data then return end
    TidyXhtml wiki_article_data  # tidy it into the cache folder, if it's not already there
    MakeXhtml wiki_article_data  # transform it by removing nav, links, etc
    GetXhtml wiki_article_data   # read raw content into the datastructure
end

def TidyXhtml(wiki_article_data)
    if nil == wiki_article_data then return end
    return if File.exists?(@cache_location_xml + "/" + File.basename(wiki_article_data.original_filename))
    #print "Making XHTML file\n"
    output_file = @cache_location_xml + "/" + File.basename(wiki_article_data.original_filename)
    command_line = @tidy_location + " -utf8 -asxhtml -quiet " + 
        "-output \"" + output_file + "\" " + 
        "\"" + wiki_article_data.base_folder + "/" + wiki_article_data.original_filename + "\""
    File.unlink("temp.xhtml") if File.exists?("temp.xhtml")
    #print "#{command_line}\n"
    # sh and bash are subtly different on the Mac.  Force bash so we can get fancy stdout redirects.
    system("bash -c '" + command_line + " > /dev/null 2>&1'")
=begin
    # This popen3 code sometimes locks up... but usually only after about 600 successful runs!
    Open3.popen3(command_line) { |stdin, stdout, stderr|
        while !stdout.eof?
            x = stdout.readlines
            #print x
        end
        while !stderr.eof?
            x = stderr.readlines
            #print x
        end
        stdin.close
        stdout.close
        stderr.close
    }
=end
    #print "BACK!\n"
    raise "Command \"" + command_line + "\" failed to produce an output file!" if !File.exists?(output_file)
end
private :TidyXhtml

def MakeXhtml(wiki_article_data)
    formatter = REXML::Formatters::Default.new
    input_file = @cache_location_xml + "/" + File.basename(wiki_article_data.original_filename)
    output_file = @cache_location_html + "/" + File.basename(wiki_article_data.original_filename)
    return if File.exists?(output_file)
    doc = REXML::Document.new(File.new(input_file, "r"))
    panes = doc.get_elements("//div[@class='pane big']")
    raise "Unable to find main content in #{wiki_article_data.original_filename}" if (panes.length != 1)
    pane = panes[0]
    titles = pane.get_elements("h3")
    raise "Unable to find title in #{wiki_article_data.original_filename}" if (titles.length == 0)
    wiki_article_data.title = FixCase(titles[0].text)
    titles[0].text = wiki_article_data.title
    
    # trim out boilerplate help text
    help_nodes = pane.get_elements("//div[@id='help_1']")
    help_nodes.each { |node| pane.delete_element node }
    help_nodes = pane.get_elements("//div[@id='help_2']")
    help_nodes.each { |node| pane.delete_element node }
    help_nodes = pane.get_elements("div")
    help_nodes.each { |node| pane.delete_element node if nil != node.text.index("If you are new to these pages")}
    
    # Remove <a> tags, grafting children of <a> up to parent of <a>
    a_nodes = pane.get_elements("//a")
    a_nodes.each { |node|
        node.children.each { |child|
            node.parent.insert_before(node, child.clone)
        }
        node.parent.delete_element node
    }
    
    # serialize article text
    output_file_object = File.new(output_file, "w")
    formatter.write(pane, output_file_object)
    output_file_object.close
end
private :MakeXhtml

def GetXhtml(wiki_article_data)
    input_file = @cache_location_html + "/" + File.basename(wiki_article_data.original_filename)
    file = File.open(input_file, "r")
    wiki_article_data.html_content = file.read
    file.close
    matches = @title_regex.match(wiki_article_data.html_content)
    wiki_article_data.title = "Untitled"
    if (matches.length > 1)
        wiki_article_data.title = matches[1]
    end
end
private :GetXhtml

def FixCase(title)
    result = title.clone
    result.gsub!("\r", " ") # remove newlines
    result.gsub!("\n", " ") # remove newlines
    result.gsub!("Viewing cable ", "") # remove static prefix
    pos = result.index(", ") # remove article name prefix
    if nil != pos
        result = result[pos+2..-1]
    end
    result.downcase!
    result.gsub!(/\b\w/){$&.upcase} # capitalize each word
    result.gsub!(/['’][ST]/){$&.downcase} # fix apostrophe-s and apostrophe-t
    return result
end
private :FixCase

end #HtmlParser
